// This is the file with the version of the model with a Taylor rule
// It produces impulse response graphs to various shocks (you have a menu of 13 shocks to select from, but only the responses to 4 are displayed. The user can change this easily.
// The program also saves impulse functions for later use by other programs.
// Note: there are some redundant parameters in this code because it was constructed by modifying the code for the much larger model constructed by Christiano, Trabant and Walentin.
// There is no need to worry about this.

// Endogenous variables
var lpmcU lpcU lqU lpxU lRU lcU lpsizplusU lpiU lsU aU lHU lwbarU
lphaloU lFdU lpitildedpiU lKdU Welf Util lyU lpicU lmarkupU lexpU limpU

// Exogenous variables
   lzetacU lmuzU lzetahU lRstarU lepsilonU lystarU lgU ltaudU lpitargetU
lpistarU phitildeU;

// shock innovations
varexo lzetacU_eps lmuzU_eps lzetahU_eps lRstarU_eps lepsilonU_eps 
lystarU_eps lgU_eps ltaudU_eps lpitargetU_eps lpistarU_eps phitildeU_eps
money_eps;

parameters nustar beta muz xid lambdad kappad vrkappad
 pibreve epsilon taud nu vrphi nux omegac etac zetac zetah AL b etag etaa
 sigmaL phitildea phitildes etaf rhoR rpi ry rdeltapi rdeltay rq 
 nbeta lambday lambdadeltaR lambdaS Welf_SS Util_SS
 lpmcU_SS lpcU_SS lqU_SS lpxU_SS lRU_SS lcU_SS lpsizplusU_SS lpiU_SS lsU_SS aU_SS lHU_SS lwbarU_SS
 lphaloU_SS lFdU_SS lpitildedpiU_SS lKdU_SS 
 lzetacU_SS lmuzU_SS lzetahU_SS lRstarU_SS lepsilonU_SS lystarU_SS lgU_SS ltaudU_SS lpitargetU_SS
 lpistarU_SS lyU_SS phitildeU_SS lmarkupU_SS lexpU_SS limpU_SS;

% Do not put ; at the end of the line
@# include "simple_openeconomy_parameters.mod"

lexpU_SS=-etaf*lpxU_SS+lystarU_SS+lpcU_SS+lqU_SS+lpxU_SS;

limpU_SS=lpmcU_SS+log(omegac)+etac*(lpcU_SS-lpmcU_SS)+lcU_SS;

// eqx denotes the equation name in the (*.tex version of the) handout on the open economy
// eqsimple x denotes the equation name in the manuscript with Karl and Matthias 

model;
//1 eqsimple 2 eq6
lpmcU-lpcU-lqU-log(nustar*exp(lRstarU)+1-nustar);

// 2 eqsimple 3 eq7
log(nux*exp(lRU)+1-nux)-lqU-lpcU-lpxU;

// 3 eqsimple 4 
exp(lzetacU)/(exp(lcU)-b*exp(lcU(-1))/exp(lmuzU))- 
beta*b*(exp(lzetacU(+1))/ 
(exp(lcU(+1))*exp(lmuzU(+1))-b*exp(lcU))) 
-exp(lpsizplusU)*exp(lpcU)=0;

// 4 eqsimple 5 eq4 
exp(lpsizplusU)-beta*exp(lpsizplusU(+1)+lRU-lpiU(+1)-lmuzU(+1));

// 5 eqsimple 6 eq3 
-exp(lpsizplusU)+beta*exp(lpsizplusU(+1))/(exp(lmuzU(+1))*exp(lpiU(+1))) 
*(exp(lsU(+1))*exp(lRstarU)* 
exp(-phitildea*(aU-aU_SS)-phitildes*(exp(lRstarU)-exp(lRU)-(exp(lRstarU_SS)-exp(lRU_SS)))+phitildeU));

// 6 eqsimple 7 
exp(lzetahU+sigmaL*lHU-lpsizplusU)*AL-exp(lwbarU);

// 7 eqsimple 8
lyU = (lambdad/(lambdad-1))*lphaloU + lepsilonU + lHU;

// 8 eqsimple 9 eq13 

aU+exp(lpmcU)*omegac*exp(etac*(lpcU-lpmcU)+lcU)
  = exp(lqU+lpcU+(1-etaf)*lpxU+lystarU) + exp(lRstarU(-1)-phitildea*(aU(-1)-aU_SS) 
  -phitildes*(exp(lRstarU(-1))-exp(lRU(-1))-(exp(lRstarU_SS)-exp(lRU_SS))) 
  +phitildeU(-1))*exp(lsU-lpiU-lmuzU)*aU(-1);  

lexpU=-etaf*lpxU+lystarU+lpcU+lqU+lpxU;

// 9 eqsimple 10 eq8
-exp(lyU) + exp(lgU) + (1-omegac)*exp(lpcU*etac+lcU) + exp(-etaf*lpxU+lystarU);

// 10 eqsimple 12 eq1
exp(lpicU)=exp(lpiU)* 
((1-omegac+omegac*(exp(lpmcU))^(1-etac))/ 
(1-omegac+omegac*(exp(lpmcU(-1)))^(1-etac)))^(1/(1-etac)); 

//Non-linear pricing equations for domestic intermediate goods producer 
//11 eq40a eq10
exp(lFdU)-exp(lpsizplusU+lyU)-beta*xid*exp(lpitildedpiU(+1)/(1-lambdad)+lFdU(+1)); 
 
//12 eq40b eq11
exp(lKdU)-lambdad*exp(lpsizplusU+lyU+ltaudU+lwbarU+log(nu*exp(lRU)+1-nu)-lepsilonU) 
-beta*xid*exp(lpitildedpiU(+1)*lambdad/(1-lambdad)+lKdU(+1)); 

//13 eq40c eq9
exp(lphaloU*lambdad/(1-lambdad))-(1-xid)*((1-xid*exp(lpitildedpiU/(1-lambdad)))/(1-xid))^lambdad 
   -xid*(exp(lpitildedpiU+lphaloU(-1)))^(lambdad/(1-lambdad)); 
 
//14 eq40d  eq11
exp(lKdU-lFdU)=((1-xid*exp(lpitildedpiU/(1-lambdad)))/(1-xid))^(1-lambdad); 
 
//15 eq40e 
exp(lpitildedpiU)=exp(-lpiU+kappad*lpiU(-1)+(1-kappad-vrkappad)*lpitargetU+vrkappad*log(pibreve)); 

// 16 eq2
// Monetary Policy Rule
//lRU-lRU_SS=rhoR*(lRU(-1)-lRU_SS)+(1-rhoR)*(rpi*(lpicU(+1)-lpitargetU_SS)+
//  ry*(lyU(+1)-lyU_SS)+rq*(lqU(+1)-lqU_SS)
//  +rdeltapi*(lpicU-lpicU(-1))+rdeltay*(lyU-lyU(-1))+money_eps;
lRU-lRU_SS=rhoR*(lRU(-1)-lRU_SS)+(1-rhoR)*(rpi*(lpicU(+1)-lpitargetU_SS)+
                 ry*(lyU(+1)-lyU_SS))+money_eps;

// 17
Util=exp(lzetacU)*log(exp(lmuzU+lcU)-b*exp(lcU(-1))) - AL*exp(lzetahU+
(1+sigmaL)*lHU)/(1+sigmaL); 
//Util=-(100*(exp(lpicU+lpicU(-1)+lpicU(-2)+lpicU(-3))-exp(4*lpitargetU)))^2 
//   - lambday*(100*(lyU-lyU_SS))^2 - lambdadeltaR*(400*(exp(lRU)-exp(lRU(-1))))^2;   

// 18 
Welf = Util + nbeta*Welf(+1);

// 19
exp(lqU)/exp(lqU(-1))=exp(lsU)*exp(lpistarU)/exp(lpicU);

// 20 eq25 eq5
exp(lpcU)-((1-omegac)+omegac*exp(lpmcU*(1-etac)))^(1/(1-etac));

//log, real marginal cost 
lmarkupU=-(ltaudU+lwbarU+log(nux*exp(lRU)+1-nux)-lepsilonU);

limpU=lpmcU+log(omegac)+etac*(lpcU-lpmcU)+lcU;

// 21 - 31
lzetacU = .95*lzetacU(-1) + lzetacU_eps;
lmuzU = (1-.95)*lmuzU_SS + .95*lmuzU(-1) + lmuzU_eps;
lzetahU = .95*lzetahU(-1) + lzetahU_eps;
lRstarU = (1-.95)*lRstarU_SS + .95*lRstarU(-1) + lRstarU_eps;
lystarU = (1-.95)*lystarU_SS + .95*lystarU(-1) + lystarU_eps;
lpistarU = (1-.95)*lpistarU_SS + .95*lpistarU(-1) + lpistarU_eps;
lepsilonU = .95*lepsilonU(-1) + lepsilonU_eps;
lgU = (1-.95)*(log(etag)+lyU_SS) + .95*lgU(-1) + lgU_eps;
//lgU = (1-1.85+0.855)*(log(etag)+lyU_SS) + 1.85*lgU(-1)-0.855*lgU(-2)+lgU_eps;

ltaudU = (1-.95)*ltaudU_SS + .95*ltaudU(-1) + ltaudU_eps;
lpitargetU = (1-.95)*lpitargetU_SS + .95*lpitargetU(-1) + lpitargetU_eps;
phitildeU = .95*phitildeU(-1) + phitildeU_eps;
end;



shocks;
var lzetacU_eps;
stderr 0.01;
var lmuzU_eps; 
stderr 0.01;
var lzetahU_eps;
stderr 0.01;
var lystarU_eps;
stderr 0.01;
var lpitargetU_eps;
stderr 0.01;
var lpistarU_eps;
stderr 0.01;
var money_eps;
stderr 0.01;
var ltaudU_eps;
stderr 0.01;
var phitildeU_eps;
stderr 0.01;
var lRstarU_eps;
stderr 0.01;
var lepsilonU_eps;
stderr 0.01;
var lgU_eps;
stderr 0.01;
end;

initval;
limpU=limpU_SS;
lexpU=lexpU_SS;
lpmcU=lpmcU_SS;
lpcU=lpcU_SS;
lqU=lpcU_SS;
lpxU=lpxU_SS;
lRU=lRU_SS;
lcU=lcU_SS;
lpsizplusU=lpsizplusU_SS;
lpitildedpiU=lpitildedpiU_SS; 
lpiU=lpiU_SS;
lsU=lsU_SS;
aU=aU_SS;
lHU=lHU_SS;
lwbarU=lwbarU_SS;
lphaloU=lphaloU_SS;
lFdU=lFdU_SS;
lKdU=lKdU_SS;
Welf=Welf_SS;
Util=Util_SS;
lyU=lyU_SS;
lpicU=lpicU_SS;
lzetacU=0;
lmuzU=lmuzU_SS;
lzetahU=0;
lRstarU=lRstarU_SS;
lystarU=lystarU_SS;
lpistarU=lpistarU_SS;
lepsilonU=0;
lgU=lgU_SS;
ltaudU=ltaudU_SS;
phitildeU = phitildeU_SS;
lpitargetU=lpitargetU_SS;
lmarkupU=lmarkupU_SS;
end;

steady;
set_dynare_seed=1;
stoch_simul(periods=5000,irf=20,order=1,nograph) lyU lcU lRU lHU lpicU lsU lqU aU Welf lpxU lexpU limpU lgU lRstarU lpiU;

//the following lines compute the UIP regression of the change in the exchange rate on the interest rate differential
y_=oo_.endo_simul;
//foreign minus domestic interest rate
ratediff=y_(24,3:end-1)-y_(23,3:end-1);
//future exchange rate minus current
dexchange=y_(25,4:end)-y_(25,3:end-1);
a=cov(dexchange,ratediff);
//regression of exchange rate change on interest rate differential
bet=a(1,2)/var(ratediff);

//following computes the relative volatility of inflation and the rate of depreciation
infl=y_(14,2:end);
exch=(y_(25,3:end-1));
rat=std(exch)/std(infl);

tt=200:275;
plot(tt,infl(tt),tt,exch(tt))
legend('inflation','exchange rate')
axis tight

//the following code can be rearranged to show impulse responses for any of the 12 shocks defined in the shocks command. 

pltout(lyU_ltaudU_eps,lcU_lgU_eps,lRU_lgU_eps,lRU_SS,lsU_lgU_eps,lHU_lgU_eps,lqU_lgU_eps,aU_lgU_eps,lpxU_lgU_eps,lpicU_lgU_eps,lpicU_SS,lexpU_lgU_eps,limpU_lgU_eps);
hout=suptitle('labor subsidy shock');
figure
pltout(lyU_lgU_eps,lcU_lgU_eps,lRU_lgU_eps,lRU_SS,lsU_lgU_eps,lHU_lgU_eps,lqU_lgU_eps,aU_lgU_eps,lpxU_lgU_eps,lpicU_lgU_eps,lpicU_SS,lexpU_lgU_eps,limpU_lgU_eps);
hout=suptitle('response to government consumption shock');
figure 
pltout(lyU_phitildeU_eps,lcU_phitildeU_eps,lRU_phitildeU_eps,lRU_SS,lsU_phitildeU_eps,lHU_phitildeU_eps,lqU_phitildeU_eps,aU_phitildeU_eps,lpxU_phitildeU_eps,lpicU_phitildeU_eps,lpicU_SS,lexpU_phitildeU_eps,limpU_phitildeU_eps);                                                             
hout=suptitle('response to country risk premium shock');
figure
load money lym lcm lRm lsm lHm lqm am lpxm lpicm lexpm limpm phitildesm
pltout(lyU_money_eps,lcU_money_eps,lRU_money_eps,lRU_SS,lsU_money_eps,lHU_money_eps,lqU_money_eps,aU_money_eps,lpxU_money_eps,lpicU_money_eps,lpicU_SS,lexpU_money_eps,limpU_money_eps,lym,lcm,lRm,lsm,lHm,lqm,am,lpxm,lpicm,lexpm,limpm);                                                                                   
hout=suptitle('response to monetary policy shock');

lym=lyU_money_eps;
lcm=lcU_money_eps;
lRm=lRU_money_eps;
lsm=lsU_money_eps;
lHm=lHU_money_eps;
lqm=lqU_money_eps;
am=aU_money_eps;
lpxm=lpxU_money_eps;
lpicm=lpicU_money_eps;
lexpm=lexpU_money_eps;
limpm=limpU_money_eps;

phitildesm=phitildes;
save money lym lcm lRm lsm lHm lqm am lpxm lpicm lexpm limpm phitildesm

//the following lines save various impulse responses for later use by other programs.

JTFWDRstar=[lyU_lRstarU_eps'
 lcU_lRstarU_eps'
 lRU_lRstarU_eps'
 lHU_lRstarU_eps'
 lpiU_lRstarU_eps'
 lsU_lRstarU_eps'
 lqU_lRstarU_eps'
 aU_lRstarU_eps'
 Welf_lRstarU_eps'+Welf_SS];

JTFWDeps=[lyU_lepsilonU_eps'
 lcU_lepsilonU_eps'
 lRU_lepsilonU_eps'
 lHU_lepsilonU_eps'
 lpiU_lepsilonU_eps'
 lsU_lepsilonU_eps'
 lqU_lepsilonU_eps'
 aU_lepsilonU_eps'
 Welf_lepsilonU_eps'+Welf_SS];

JTFWDg=[lyU_lgU_eps'
 lcU_lgU_eps'
 lRU_lgU_eps'
 lHU_lgU_eps'
 lpiU_lgU_eps'
 lsU_lgU_eps'
 lqU_lgU_eps'
 aU_lgU_eps'
 Welf_lgU_eps'+Welf_SS];

JTFWDtaud=[lyU_ltaudU_eps'
 lcU_ltaudU_eps'
 lRU_ltaudU_eps'
 lHU_ltaudU_eps'
 lpiU_ltaudU_eps'
 lsU_ltaudU_eps'
 lqU_ltaudU_eps'
 aU_ltaudU_eps'
 Welf_ltaudU_eps'+Welf_SS];

JTFWDphitilde=[lyU_phitildeU_eps'
 lcU_phitildeU_eps'
 lRU_phitildeU_eps'
 lHU_phitildeU_eps'
 lpiU_phitildeU_eps'
 lsU_phitildeU_eps'
 lqU_phitildeU_eps'
 aU_phitildeU_eps'
 Welf_phitildeU_eps'+Welf_SS];

save JT JTFWDRstar JTFWDeps JTFWDg JTFWDtaud JTFWDphitilde Welf_SS;
